/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-27 16:16:29.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.base.Constants;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.JsonMapper;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.AuditStatus;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.DisciplineType;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.SelectType;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamAuditService;
import cn.com.qmth.examcloud.core.oe.admin.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examaudit.*;
import cn.com.qmth.examcloud.core.oe.admin.service.others.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 考试记录审核相关接口
 *
 * @author: fengdesheng
 * @since: 2018/8/27
 */
@RestController
@Api(tags = "考试记录审核相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/audit")
public class ExamAuditController extends ControllerSupport {

    @Autowired
    private ExamAuditService examAuditService;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private GainBaseDataService gainBaseDataService;

    @PostMapping("/list")
    @ApiOperation(value = "查询“监考已审”列表(分页)")
    public Page<ExamAuditInfo> getExamAuditList(@RequestBody ExamAuditQuery query) {
        User user = getAccessUser();
        query.setRootOrgId(user.getRootOrgId());
        Page<ExamAuditInfo> examAuditList = examAuditService.getExamAuditList(query);

        examAuditList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(user.getRootOrgId(), p.getIdentityNumber()));
        });
        return examAuditList;
    }

    @PostMapping("/discipline/list")
    @ApiOperation(value = "查询“违纪名单”列表(分页)")
    public Page<ExamAuditInfo> getExamAuditUnPassList(@RequestBody ExamAuditQuery query) {
        query.setStatus(AuditStatus.UN_PASS.name());
        User user = getAccessUser();
        query.setRootOrgId(user.getRootOrgId());
        Page<ExamAuditInfo> examAuditList = examAuditService.getExamAuditList(query);

        examAuditList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(user.getRootOrgId(), p.getIdentityNumber()));
        });
        return examAuditList;
    }

    @Naked
    @GetMapping("/discipline/list/export")
    @ApiOperation(value = "导出“违纪名单”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamAuditUnPassList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamAuditQuery newQuery = new JsonMapper().fromJson(query, ExamAuditQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");

        newQuery.setStatus(AuditStatus.UN_PASS.name());
        newQuery.setSelectType(SelectType.EXPORT);
        Page<ExamAuditInfo> page = examAuditService.getExamAuditList(newQuery);
        List<ExamAuditExcel> list = ExamAuditEntityConvert.ofExcel(page);
        ExportUtils.exportEXCEL("违纪名单列表", ExamAuditExcel.class, list, response);
    }

    @PostMapping(value = "/single/audit")
    @ApiOperation(value = "监考待审-单个审核")
    public void singleAudit(@RequestParam(required = true) Long examRecordDataId,
                            @RequestParam(required = false) String disciplineDetail,
                            @RequestParam(required = true) Boolean isPass,
                            @RequestParam(required = false) DisciplineType disciplineType) {
        if (examRecordDataId == null) {
            throw new StatusException("singleAudit-001", "examRecordDataId不能为空");
        }
        if (isPass == null) {
            throw new StatusException("singleAudit-002", "isPass不能为空");
        }
        User user = getAccessUser();
        examAuditService.singleAudit(examRecordDataId, isPass, disciplineDetail, disciplineType, user);
    }

    @PostMapping(value = "/batch/audit")
    @ApiOperation(value = "监考待审-批量审核")
    public void batchAudit(@RequestParam List<Long> examRecordDataIds, @RequestParam Boolean isPass) {
        if (examRecordDataIds == null || examRecordDataIds.size() == 0) {
            throw new StatusException("batchAudit-001", "examRecordDataIds不能为空");
        }
        if (isPass == null) {
            throw new StatusException("batchAudit-002", "isPass不能为空");
        }
        User user = getAccessUser();
        examAuditService.batchAudit(examRecordDataIds, isPass, user);
    }

    @PostMapping(value = "/redoAudit")
    @ApiOperation(value = "重新审核")
    public void redoAudit(@RequestBody RedoAuditInfo redoAuditInfo) {
        User user = getAccessUser();
        if (redoAuditInfo.getExamRecordDataIds() == null || redoAuditInfo.getExamRecordDataIds().size() == 0) {
            throw new StatusException("redoAudit-001", "examRecordDataIds不能为空");
        }
        if (redoAuditInfo.getIsPass() == null) {
            throw new StatusException("redoAudit-002", "isPass不能为空");
        }
        if (!redoAuditInfo.getIsPass() && StringUtils.isBlank(redoAuditInfo.getDisciplineType())) {
            throw new StatusException("redoAudit-003", "审核为不通过时违纪类型不能为空");
        }
        Set<Long> examIds = new HashSet<Long>();
        for (Long examRecordDataId : redoAuditInfo.getExamRecordDataIds()) {
            ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
            examIds.add(examRecordData.getExamId().longValue());
        }
        if (examIds.size() > 1) {
            throw new StatusException("redoAudit-004", "考试ID中包含了多个批次");
        }
        long examId = examIds.iterator().next();

        //原需求：已经生成了评卷任务，不能重新审为不通过，但是可以审核为已通过
        //20191220,需求临时调整：已经生成了评卷任务,均不可重审
        String markingTaskBuilt =
                ExamCacheTransferHelper.getDefaultCachedExamProperty(examId, ExamProperties.MARKING_TASK_BUILDED.name()).getValue();
        if (StringUtils.isNotBlank(markingTaskBuilt) && Constants.isTrue.equals(markingTaskBuilt)) {
//            if (!redoAuditInfo.getIsPass()) {
            throw new StatusException("redoAudit-005", "该考试评卷任务已生成,不允许重审");
//            }
        }
        examAuditService.redoAudit(redoAuditInfo, user);
    }

}
